@@ -58,8 +58,6 @@
 #        include "libANGLE/renderer/gl/wgl/DisplayWGL.h"
 #    elif ANGLE_ENABLE_CGL
 #        include "libANGLE/renderer/gl/cgl/DisplayCGL.h"
-#    elif ANGLE_ENABLE_EAGL
-#        include "libANGLE/renderer/gl/eagl/DisplayEAGL.h"
 #    elif defined(ANGLE_PLATFORM_LINUX)
 #        include "libANGLE/renderer/gl/egl/DisplayEGL.h"
 #        if defined(ANGLE_USE_X11)
@@ -222,6 +220,18 @@ inline bool operator==(const ANGLEPlatformDisplay &a, const ANGLEPlatformDisplay
     return a.tie() == b.tie();
 }
 
+static angle::SimpleMutex *DevicePlatformDisplayMapMutex()
+{
+    static angle::base::NoDestructor<angle::SimpleMutex> devicePlatformDisplayMapMutex;
+    return devicePlatformDisplayMapMutex.get();
+}
+
+static angle::SimpleMutex *ANGLEPlatformDisplayMapMutex()
+{
+    static angle::base::NoDestructor<angle::SimpleMutex> anglePlatformDisplayMapMutex;
+    return anglePlatformDisplayMapMutex.get();
+}
+
 static constexpr size_t kANGLEPlatformDisplayMapSize = 9;
 typedef angle::FlatUnorderedMap<ANGLEPlatformDisplay, Display *, kANGLEPlatformDisplayMapSize>
     ANGLEPlatformDisplayMap;
@@ -412,10 +422,6 @@ rx::DisplayImpl *CreateDisplayFromAttribs(EGLAttrib displayType,
             impl = new rx::DisplayCGL(state);
             break;
 
-#    elif ANGLE_ENABLE_EAGL
-            impl = new rx::DisplayEAGL(state);
-            break;
-
 #    elif defined(ANGLE_PLATFORM_LINUX)
 #        if defined(ANGLE_USE_GBM)
             if (platformType == 0)
@@ -717,9 +723,12 @@ void DisplayState::notifyDeviceLost() const
         return;
     }
 
-    for (auto context = contextMap.begin(); context != contextMap.end(); context++)
     {
-        context->second->markContextLost(gl::GraphicsResetStatus::UnknownContextReset);
+        std::lock_guard<angle::SimpleMutex> lock(contextMapMutex);
+        for (auto context = contextMap.begin(); context != contextMap.end(); context++)
+        {
+            context->second->markContextLost(gl::GraphicsResetStatus::UnknownContextReset);
+        }
     }
 
     deviceLost = true;
@@ -782,27 +791,32 @@ Display *Display::GetDisplayFromNativeDisplay(EGLenum platform,
         updatedAttribMap.get(EGL_FEATURE_OVERRIDES_DISABLED_ANGLE, 0);
     EGLAttrib disableAllNonOverriddenFeatures =
         updatedAttribMap.get(EGL_FEATURE_ALL_DISABLED_ANGLE, 0);
-    ANGLEPlatformDisplayMap *displays = GetANGLEPlatformDisplayMap();
     ANGLEPlatformDisplay combinedDisplayKey(
         nativeDisplay, powerPreference, platformANGLEType, deviceIdHigh, deviceIdLow, displayKey,
         enabledFeatureOverrides, disabledFeatureOverrides, disableAllNonOverriddenFeatures);
-    const auto &iter = displays->find(combinedDisplayKey);
 
-    if (iter != displays->end())
     {
-        display = iter->second;
-    }
+        std::lock_guard<angle::SimpleMutex> lock(*ANGLEPlatformDisplayMapMutex());
 
-    if (display == nullptr)
-    {
-        // Validate the native display
-        if (!Display::isValidNativeDisplay(nativeDisplay))
+        ANGLEPlatformDisplayMap *displays = GetANGLEPlatformDisplayMap();
+        const auto &iter                  = displays->find(combinedDisplayKey);
+
+        if (iter != displays->end())
         {
-            return nullptr;
+            display = iter->second;
         }
 
-        display = new Display(platform, nativeDisplay, nullptr);
-        displays->insert(std::make_pair(combinedDisplayKey, display));
+        if (display == nullptr)
+        {
+            // Validate the native display
+            if (!Display::isValidNativeDisplay(nativeDisplay))
+            {
+                return nullptr;
+            }
+
+            display = new Display(platform, nativeDisplay, nullptr);
+            displays->insert(std::make_pair(combinedDisplayKey, display));
+        }
     }
     // Apply new attributes if the display is not initialized yet.
     if (!display->isInitialized())
@@ -838,6 +852,7 @@ Display *Display::GetDisplayFromNativeDisplay(EGLenum platform,
 // static
 Display *Display::GetExistingDisplayFromNativeDisplay(EGLNativeDisplayType nativeDisplay)
 {
+    std::lock_guard<angle::SimpleMutex> lock(*ANGLEPlatformDisplayMapMutex());
     ANGLEPlatformDisplayMap *displays = GetANGLEPlatformDisplayMap();
     const auto &iter                  = displays->find(nativeDisplay);
 
@@ -857,34 +872,39 @@ Display *Display::GetDisplayFromDevice(Device *device, const AttributeMap &attri
 
     ASSERT(Device::IsValidDevice(device));
 
-    ANGLEPlatformDisplayMap *anglePlatformDisplays   = GetANGLEPlatformDisplayMap();
-    DevicePlatformDisplayMap *devicePlatformDisplays = GetDevicePlatformDisplayMap();
-
-    // First see if this eglDevice is in use by a Display created using ANGLE platform
-    for (auto &displayMapEntry : *anglePlatformDisplays)
     {
-        egl::Display *iterDisplay = displayMapEntry.second;
-        if (iterDisplay->getDevice() == device)
+        // First see if this eglDevice is in use by a Display created using ANGLE platform
+        std::lock_guard<angle::SimpleMutex> lock(*ANGLEPlatformDisplayMapMutex());
+        ANGLEPlatformDisplayMap *anglePlatformDisplays = GetANGLEPlatformDisplayMap();
+        for (auto &displayMapEntry : *anglePlatformDisplays)
         {
-            display = iterDisplay;
+            egl::Display *iterDisplay = displayMapEntry.second;
+            if (iterDisplay->getDevice() == device)
+            {
+                display = iterDisplay;
+            }
         }
     }
 
     if (display == nullptr)
     {
+        // Next see if this eglDevice is in use by a Display created using the DEVICE platform
+        std::lock_guard<angle::SimpleMutex> lock(*DevicePlatformDisplayMapMutex());
+        DevicePlatformDisplayMap *devicePlatformDisplays = GetDevicePlatformDisplayMap();
+
         // See if the eglDevice is in use by a Display created using the DEVICE platform
         const auto &iter = devicePlatformDisplays->find(device);
         if (iter != devicePlatformDisplays->end())
         {
             display = iter->second;
         }
-    }
 
-    if (display == nullptr)
-    {
-        // Otherwise create a new Display
-        display = new Display(EGL_PLATFORM_DEVICE_EXT, 0, device);
-        devicePlatformDisplays->insert(std::make_pair(device, display));
+        if (display == nullptr)
+        {
+            // Otherwise create a new Display
+            display = new Display(EGL_PLATFORM_DEVICE_EXT, 0, device);
+            devicePlatformDisplays->insert(std::make_pair(device, display));
+        }
     }
 
     // Apply new attributes if the display is not initialized yet.
@@ -898,27 +918,6 @@ Display *Display::GetDisplayFromDevice(Device *device, const AttributeMap &attri
     return display;
 }
 
-// static
-Display::EglDisplaySet Display::GetEglDisplaySet()
-{
-    Display::EglDisplaySet displays;
-
-    ANGLEPlatformDisplayMap *anglePlatformDisplays   = GetANGLEPlatformDisplayMap();
-    DevicePlatformDisplayMap *devicePlatformDisplays = GetDevicePlatformDisplayMap();
-
-    for (auto anglePlatformDisplayMapEntry : *anglePlatformDisplays)
-    {
-        displays.insert(anglePlatformDisplayMapEntry.second);
-    }
-
-    for (auto devicePlatformDisplayMapEntry : *devicePlatformDisplays)
-    {
-        displays.insert(devicePlatformDisplayMapEntry.second);
-    }
-
-    return displays;
-}
-
 Display::Display(EGLenum platform, EGLNativeDisplayType displayId, Device *eglDevice)
     : mState(displayId),
       mImplementation(nullptr),
@@ -960,6 +959,7 @@ Display::~Display()
         case EGL_PLATFORM_WAYLAND_EXT:
         case EGL_PLATFORM_SURFACELESS_MESA:
         {
+            std::lock_guard<angle::SimpleMutex> lock(*ANGLEPlatformDisplayMapMutex());
             ANGLEPlatformDisplayMap *displays      = GetANGLEPlatformDisplayMap();
             ANGLEPlatformDisplayMap::iterator iter = displays->find(ANGLEPlatformDisplay(
                 mState.displayId,
@@ -980,6 +980,7 @@ Display::~Display()
         }
         case EGL_PLATFORM_DEVICE_EXT:
         {
+            std::lock_guard<angle::SimpleMutex> lock(*DevicePlatformDisplayMapMutex());
             DevicePlatformDisplayMap *displays      = GetDevicePlatformDisplayMap();
             DevicePlatformDisplayMap::iterator iter = displays->find(mDevice);
             if (iter != displays->end())
@@ -1012,6 +1013,7 @@ void Display::onSubjectStateChange(angle::SubjectIndex index, angle::SubjectMess
 {
     ASSERT(index == kGPUSwitchedSubjectIndex);
     ASSERT(message == angle::SubjectMessage::SubjectChanged);
+    std::lock_guard<angle::SimpleMutex> lock(mState.contextMapMutex);
     for (auto context : mState.contextMap)
     {
         context.second->onGPUSwitch();
@@ -1100,11 +1102,6 @@ Error Display::initialize()
         // config.second.conformant |= EGL_OPENGL_ES_BIT;
 
         config.second.renderableType |= EGL_OPENGL_ES_BIT;
-
-        // If we aren't using desktop GL entry points, remove desktop GL support from all configs
-#if !defined(ANGLE_ENABLE_GL_DESKTOP_FRONTEND)
-        config.second.renderableType &= ~EGL_OPENGL_BIT;
-#endif
     }
 
     mFrontendFeatures.reset();
@@ -1169,6 +1166,11 @@ Error Display::initialize()
 Error Display::destroyInvalidEglObjects()
 {
     // Destroy invalid EGL objects
+    // Note that we don't need to lock mState.contextMapMutex here.
+    // Write and read access to mInvalidContextMap are coming from
+    // EGL_Terminate, EGL_ReleaseThread, ThreadCleanupCallBACK.
+    // Those functions are protected by egl global lock,
+    // so there is no race condition on mInvalidContextMap.
     while (!mInvalidContextMap.empty())
     {
         gl::Context *context = mInvalidContextMap.begin()->second;
@@ -1177,7 +1179,7 @@ Error Display::destroyInvalidEglObjects()
         // Need AddRefLock because there may be ContextMutex destruction.
         ScopedContextMutexAddRefLock lock(context->getContextMutex());
         context->setIsDestroyed();
-        ANGLE_TRY(releaseContextImpl(context, &mInvalidContextMap));
+        ANGLE_TRY(releaseContextImpl(eraseContextImpl(context, &mInvalidContextMap)));
     }
 
     while (!mInvalidImageMap.empty())
@@ -1205,6 +1207,7 @@ Error Display::destroyInvalidEglObjects()
 
 Error Display::terminate(Thread *thread, TerminateReason terminateReason)
 {
+
     if (terminateReason == TerminateReason::Api)
     {
         mTerminatedByApi = true;
@@ -1220,13 +1223,13 @@ Error Display::terminate(Thread *thread, TerminateReason terminateReason)
     // EGL 1.5 Specification
     // 3.2 Initialization
     // Termination marks all EGL-specific resources, such as contexts and surfaces, associated
-    // with the specified display for deletion. Handles to all such resources are invalid as soon
-    // as eglTerminate returns. Cache EGL objects that are no longer valid.
+    // with the specified display for deletion. Handles to all such resources are invalid as
+    // soon as eglTerminate returns. Cache EGL objects that are no longer valid.
     //
     // It is fairly common for apps to call eglTerminate while some contexts and/or surfaces are
-    // still current on some thread. Since objects are refCounted, trying to destroy them right away
-    // would only result in a decRef. We instead cache such invalid objects and use other EGL
-    // entrypoints like eglReleaseThread or thread exit events (on the Android platform) to
+    // still current on some thread. Since objects are refCounted, trying to destroy them right
+    // away would only result in a decRef. We instead cache such invalid objects and use other
+    // EGL entrypoints like eglReleaseThread or thread exit events (on the Android platform) to
     // perform the necessary cleanup.
     mInvalidImageMap.insert(mImageMap.begin(), mImageMap.end());
     mImageMap.clear();
@@ -1241,42 +1244,50 @@ Error Display::terminate(Thread *thread, TerminateReason terminateReason)
                            std::make_move_iterator(mSyncMap.end()));
     mSyncMap.clear();
 
-    // Cache total number of contexts before invalidation. This is used as a check to verify that
-    // no context is "lost" while being moved between the various sets.
-    size_t contextSetSizeBeforeInvalidation = mState.contextMap.size() + mInvalidContextMap.size();
-
-    // If app called eglTerminate and no active threads remain,
-    // force release any context that is still current.
-    ContextMap contextsStillCurrent = {};
-    for (auto context : mState.contextMap)
     {
-        if (context.second->isReferenced())
+        // Lock mState.contextMapMutex to protect mState.contextMap.
+        // mInvalidContextMap does not need protection. It just happens to fall within this scope.
+        std::lock_guard<angle::SimpleMutex> lock(mState.contextMapMutex);
+        // Cache total number of contexts before invalidation. This is used as a check to verify
+        // that no context is "lost" while being moved between the various sets.
+        size_t contextSetSizeBeforeInvalidation =
+            mState.contextMap.size() + mInvalidContextMap.size();
+
+        // If app called eglTerminate and no active threads remain,
+        // force release any context that is still current.
+        ContextMap contextsStillCurrent = {};
+        for (auto context : mState.contextMap)
         {
-            contextsStillCurrent.emplace(context);
-            continue;
+            if (context.second->isReferenced())
+            {
+                contextsStillCurrent.emplace(context);
+                continue;
+            }
+
+            // Add context that is not current to mInvalidContextSet for cleanup.
+            mInvalidContextMap.emplace(context);
         }
 
-        // Add context that is not current to mInvalidContextSet for cleanup.
-        mInvalidContextMap.emplace(context);
-    }
+        // There are many methods that require contexts that are still current to be present in
+        // display's contextSet like during context release or to notify of state changes in a
+        // subject. So as to not interrupt this flow, do not remove contexts that are still
+        // current on some thread from display's contextSet even though eglTerminate marks such
+        // contexts as invalid.
+        //
+        // "mState.contextSet" will now contain only those contexts that are still current on
+        // some thread.
+        mState.contextMap = std::move(contextsStillCurrent);
 
-    // There are many methods that require contexts that are still current to be present in
-    // display's contextSet like during context release or to notify of state changes in a subject.
-    // So as to not interrupt this flow, do not remove contexts that are still current on some
-    // thread from display's contextSet even though eglTerminate marks such contexts as invalid.
-    //
-    // "mState.contextSet" will now contain only those contexts that are still current on some
-    // thread.
-    mState.contextMap = std::move(contextsStillCurrent);
+        // Assert that the total number of contexts is the same before and after context
+        // invalidation.
+        ASSERT(contextSetSizeBeforeInvalidation ==
+               mState.contextMap.size() + mInvalidContextMap.size());
 
-    // Assert that the total number of contexts is the same before and after context invalidation.
-    ASSERT(contextSetSizeBeforeInvalidation ==
-           mState.contextMap.size() + mInvalidContextMap.size());
-
-    if (!mState.contextMap.empty())
-    {
-        // There was atleast 1 context that was current on some thread, early return.
-        return NoError();
+        if (!mState.contextMap.empty())
+        {
+            // There was atleast 1 context that was current on some thread, early return.
+            return NoError();
+        }
     }
 
     // The global texture and semaphore managers should be deleted with the last context that uses
@@ -1551,7 +1562,6 @@ Error Display::createStream(const AttributeMap &attribs, Stream **outStream)
 
 Error Display::createContext(const Config *configuration,
                              gl::Context *shareContext,
-                             EGLenum clientType,
                              const AttributeMap &attribs,
                              gl::Context **outContext)
 {
@@ -1641,8 +1651,8 @@ Error Display::createContext(const Config *configuration,
 
     gl::Context *context =
         new gl::Context(this, configuration, shareContext, shareTextures, shareSemaphores,
-                        sharedContextMutex, programCachePointer, shaderCachePointer, clientType,
-                        attribs, mDisplayExtensions, GetClientExtensions());
+                        sharedContextMutex, programCachePointer, shaderCachePointer, attribs,
+                        mDisplayExtensions, GetClientExtensions());
     Error error = context->initialize();
     if (error.isError())
     {
@@ -1656,7 +1666,10 @@ Error Display::createContext(const Config *configuration,
     }
 
     ASSERT(context != nullptr);
-    mState.contextMap.insert(std::pair(context->id().value, context));
+    {
+        std::lock_guard<angle::SimpleMutex> lock(mState.contextMapMutex);
+        mState.contextMap.insert(std::pair(context->id().value, context));
+    }
 
     ASSERT(outContext != nullptr);
     *outContext = context;
@@ -1778,14 +1791,17 @@ Error Display::makeCurrent(Thread *thread,
 
 Error Display::restoreLostDevice()
 {
-    for (ContextMap::iterator ctx = mState.contextMap.begin(); ctx != mState.contextMap.end();
-         ctx++)
     {
-        if (ctx->second->isResetNotificationEnabled())
+        std::lock_guard<angle::SimpleMutex> lock(mState.contextMapMutex);
+        for (ContextMap::iterator ctx = mState.contextMap.begin(); ctx != mState.contextMap.end();
+             ctx++)
         {
-            // If reset notifications have been requested, application must delete all contexts
-            // first
-            return EglContextLost();
+            if (ctx->second->isResetNotificationEnabled())
+            {
+                // If reset notifications have been requested, application must delete all contexts
+                // first
+                return EglContextLost();
+            }
         }
     }
 
@@ -1847,10 +1863,16 @@ void Display::destroyStreamImpl(Stream *stream, StreamSet *streams)
 // as part of destruction.
 Error Display::releaseContext(gl::Context *context, Thread *thread)
 {
-    return releaseContextImpl(context, &mState.contextMap);
+    // Use scoped_ptr to make sure the context is always freed.
+    std::unique_ptr<gl::Context> uniqueContextPtr;
+    {
+        std::lock_guard<angle::SimpleMutex> lock(mState.contextMapMutex);
+        uniqueContextPtr = eraseContextImpl(context, &mState.contextMap);
+    }
+    return releaseContextImpl(std::move(uniqueContextPtr));
 }
 
-Error Display::releaseContextImpl(gl::Context *context, ContextMap *contexts)
+std::unique_ptr<gl::Context> Display::eraseContextImpl(gl::Context *context, ContextMap *contexts)
 {
     ASSERT(!context->isReferenced());
 
@@ -1859,6 +1881,11 @@ Error Display::releaseContextImpl(gl::Context *context, ContextMap *contexts)
     ASSERT(contexts->find(context->id().value) != contexts->end());
     contexts->erase(context->id().value);
 
+    return unique_context;
+}
+
+Error Display::releaseContextImpl(std::unique_ptr<gl::Context> &&context)
+{
     if (context->usingDisplayTextureShareGroup())
     {
         ASSERT(mGlobalTextureShareGroupUsers >= 1 && mTextureManager != nullptr);
@@ -1867,7 +1894,7 @@ Error Display::releaseContextImpl(gl::Context *context, ContextMap *contexts)
             // If this is the last context using the global share group, destroy the global
             // texture manager so that the textures can be destroyed while a context still
             // exists
-            mTextureManager->release(context);
+            mTextureManager->release(context.get());
             mTextureManager = nullptr;
         }
         mGlobalTextureShareGroupUsers--;
@@ -1881,7 +1908,7 @@ Error Display::releaseContextImpl(gl::Context *context, ContextMap *contexts)
             // If this is the last context using the global share group, destroy the global
             // semaphore manager so that the semaphores can be destroyed while a context still
             // exists
-            mSemaphoreManager->release(context);
+            mSemaphoreManager->release(context.get());
             mSemaphoreManager = nullptr;
         }
         mGlobalSemaphoreShareGroupUsers--;
@@ -1928,7 +1955,7 @@ Error Display::destroyContext(Thread *thread, gl::Context *context)
 
         // Make the context current, so we can release resources belong to the context, and then
         // when context is released from the current, it will be destroyed.
-        // TODO(http://www.anglebug.com/42264840): Don't require a Context to be current in order to
+        // TODO(http://anglebug.com/42264840): Don't require a Context to be current in order to
         // destroy it.
         ANGLE_TRY(makeCurrent(thread, currentContext, nullptr, nullptr, context));
         ANGLE_TRY(
@@ -2141,8 +2168,9 @@ static ClientExtensions GenerateClientExtensions()
 #endif
 
 #if defined(ANGLE_ENABLE_VULKAN)
-    extensions.platformANGLEVulkan   = true;
-    extensions.platformANGLEDeviceId = true;
+    extensions.platformANGLEVulkan           = true;
+    extensions.platformANGLEVulkanDeviceUUID = true;
+    extensions.platformANGLEDeviceId         = true;
 #endif
 
 #if defined(ANGLE_ENABLE_SWIFTSHADER)
@@ -2162,10 +2190,6 @@ static ClientExtensions GenerateClientExtensions()
     extensions.platformANGLEDeviceTypeEGLANGLE = true;
 #endif
 
-#if defined(ANGLE_ENABLE_EAGL)
-    extensions.platformANGLEDeviceContextVolatileEagl = true;
-#endif
-
 #if defined(ANGLE_ENABLE_CGL)
     extensions.platformANGLEDeviceContextVolatileCgl = true;
 #endif
@@ -2241,6 +2265,9 @@ void Display::initDisplayExtensions()
     // All backends support specific context versions
     mDisplayExtensions.createContextBackwardsCompatible = true;
 
+    // EGL_ANGLE_memory_usage_report is implemented on front end.
+    mDisplayExtensions.memoryUsageReportANGLE = true;
+
     mDisplayExtensionString = GenerateExtensionsString(mDisplayExtensions);
 }
 
@@ -2274,21 +2301,27 @@ Error Display::valdiatePixmap(const Config *config,
 
 bool Display::isValidDisplay(const egl::Display *display)
 {
-    const ANGLEPlatformDisplayMap *anglePlatformDisplayMap = GetANGLEPlatformDisplayMap();
-    for (const auto &displayPair : *anglePlatformDisplayMap)
     {
-        if (displayPair.second == display)
+        std::lock_guard<angle::SimpleMutex> lock(*ANGLEPlatformDisplayMapMutex());
+        const ANGLEPlatformDisplayMap *anglePlatformDisplayMap = GetANGLEPlatformDisplayMap();
+        for (const auto &displayPair : *anglePlatformDisplayMap)
         {
-            return true;
+            if (displayPair.second == display)
+            {
+                return true;
+            }
         }
     }
 
-    const DevicePlatformDisplayMap *devicePlatformDisplayMap = GetDevicePlatformDisplayMap();
-    for (const auto &displayPair : *devicePlatformDisplayMap)
     {
-        if (displayPair.second == display)
+        std::lock_guard<angle::SimpleMutex> lock(*DevicePlatformDisplayMapMutex());
+        const DevicePlatformDisplayMap *devicePlatformDisplayMap = GetDevicePlatformDisplayMap();
+        for (const auto &displayPair : *devicePlatformDisplayMap)
         {
-            return true;
+            if (displayPair.second == display)
+            {
+                return true;
+            }
         }
     }
 
@@ -2332,18 +2365,7 @@ void Display::initVersionString()
 
 void Display::initClientAPIString()
 {
-    std::string supportedClientAPIs = "OpenGL_ES";
-
-#ifdef ANGLE_ENABLE_GL_DESKTOP_FRONTEND
-    // If angle_enable_gl_desktop_frontend is enabled and the max supported desktop version
-    // is not None, we support a desktop GL frontend.
-    if (mImplementation->getMaxSupportedDesktopVersion().valid())
-    {
-        supportedClientAPIs += " OpenGL";
-    }
-#endif  // ANGLE_ENABLE_GL_DESKTOP_FRONTEND
-
-    mClientAPIString = supportedClientAPIs;
+    mClientAPIString = "OpenGL_ES";
 }
 
 void Display::initializeFrontendFeatures()
@@ -2534,18 +2556,9 @@ const char *Display::queryStringi(const EGLint name, const EGLint index)
         case EGL_FEATURE_CATEGORY_ANGLE:
             result = angle::FeatureCategoryToString(mFeatures[index]->category);
             break;
-        case EGL_FEATURE_DESCRIPTION_ANGLE:
-            result = mFeatures[index]->description;
-            break;
-        case EGL_FEATURE_BUG_ANGLE:
-            result = mFeatures[index]->bug;
-            break;
         case EGL_FEATURE_STATUS_ANGLE:
             result = angle::FeatureStatusToString(mFeatures[index]->enabled);
             break;
-        case EGL_FEATURE_CONDITION_ANGLE:
-            result = mFeatures[index]->condition;
-            break;
         default:
             UNREACHABLE();
             return nullptr;
@@ -2633,6 +2646,16 @@ Error Display::waitUntilWorkScheduled()
     return NoError();
 }
 
+void Display::lockVulkanQueue()
+{
+    return mImplementation->lockVulkanQueue();
+}
+
+void Display::unlockVulkanQueue()
+{
+    return mImplementation->unlockVulkanQueue();
+}
+
 bool Display::supportsDmaBufFormat(EGLint format) const
 {
     return mImplementation->supportsDmaBufFormat(format);
@@ -2655,6 +2678,18 @@ Error Display::queryDmaBufModifiers(EGLint format,
     return NoError();
 }
 
+Error Display::querySupportedCompressionRates(const Config *configuration,
+                                              const AttributeMap &attributes,
+                                              EGLint *rates,
+                                              EGLint rate_size,
+                                              EGLint *num_rates) const
+{
+    ANGLE_TRY(mImplementation->querySupportedCompressionRates(configuration, attributes, rates,
+                                                              rate_size, num_rates));
+
+    return NoError();
+}
+
 angle::ImageLoadContext Display::getImageLoadContext() const
 {
     angle::ImageLoadContext imageLoadContext;
@@ -2669,6 +2704,7 @@ angle::ImageLoadContext Display::getImageLoadContext() const
 
 const gl::Context *Display::getContext(gl::ContextID contextID) const
 {
+    std::lock_guard<angle::SimpleMutex> lock(mState.contextMapMutex);
     auto iter = mState.contextMap.find(contextID.value);
     return iter != mState.contextMap.end() ? iter->second : nullptr;
 }
@@ -2693,6 +2729,7 @@ const egl::Sync *Display::getSync(egl::SyncID syncID) const
 
 gl::Context *Display::getContext(gl::ContextID contextID)
 {
+    std::lock_guard<angle::SimpleMutex> lock(mState.contextMapMutex);
     auto iter = mState.contextMap.find(contextID.value);
     return iter != mState.contextMap.end() ? iter->second : nullptr;
 }
