@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 Apple Inc. All rights reserved.
+ * Copyright (C) 2015-2024 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,23 +30,22 @@
 #include "IDBResultData.h"
 #include "Logging.h"
 #include "MemoryIDBBackingStore.h"
-#include "SQLiteDatabase.h"
-#include "SQLiteDatabaseTracker.h"
 #include "SQLiteFileSystem.h"
 #include "SQLiteIDBBackingStore.h"
-#include "SQLiteStatement.h"
 #include "SecurityOrigin.h"
-#include "StorageQuotaManager.h"
-#include <wtf/CrossThreadCopier.h>
+#include <wtf/CompletionHandler.h>
 #include <wtf/Locker.h>
 #include <wtf/MainThread.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 namespace IDBServer {
 
-IDBServer::IDBServer(PAL::SessionID sessionID, const String& databaseDirectoryPath, StorageQuotaManagerSpaceRequester&& spaceRequester, Lock& lock)
-    : m_sessionID(sessionID)
-    , m_spaceRequester(WTFMove(spaceRequester))
+WTF_MAKE_TZONE_ALLOCATED_IMPL(IDBBackingStore);
+WTF_MAKE_TZONE_ALLOCATED_IMPL(IDBServer);
+
+IDBServer::IDBServer(const String& databaseDirectoryPath, SpaceRequester&& spaceRequester, Lock& lock)
+    : m_spaceRequester(WTFMove(spaceRequester))
     , m_lock(lock)
 {
     ASSERT(!isMainThread());
@@ -130,14 +129,13 @@ std::unique_ptr<IDBBackingStore> IDBServer::createBackingStore(const IDBDatabase
     if (m_databaseDirectoryPath.isEmpty())
         return makeUnique<MemoryIDBBackingStore>(identifier);
 
-    ASSERT(!m_sessionID.isEphemeral());
     if (identifier.isTransient())
         return makeUnique<MemoryIDBBackingStore>(identifier);
 
     return makeUnique<SQLiteIDBBackingStore>(identifier, upgradedDatabaseDirectory(identifier));
 }
 
-void IDBServer::openDatabase(const IDBRequestData& requestData)
+void IDBServer::openDatabase(const IDBOpenRequestData& requestData)
 {
     LOG(IndexedDB, "IDBServer::openDatabase");
     ASSERT(!isMainThread());
@@ -145,7 +143,11 @@ void IDBServer::openDatabase(const IDBRequestData& requestData)
 
     auto& uniqueIDBDatabase = getOrCreateUniqueIDBDatabase(requestData.databaseIdentifier());
 
-    auto connection = m_connectionMap.get(requestData.requestIdentifier().connectionIdentifier());
+    auto connectionIdentifier = requestData.requestIdentifier().connectionIdentifier();
+    if (!connectionIdentifier)
+        return;
+
+    RefPtr connection = m_connectionMap.get(*connectionIdentifier);
     if (!connection) {
         // If the connection back to the client is gone, there's no way to open the database as
         // well as no way to message back failure.
@@ -155,13 +157,17 @@ void IDBServer::openDatabase(const IDBRequestData& requestData)
     uniqueIDBDatabase.openDatabaseConnection(*connection, requestData);
 }
 
-void IDBServer::deleteDatabase(const IDBRequestData& requestData)
+void IDBServer::deleteDatabase(const IDBOpenRequestData& requestData)
 {
     LOG(IndexedDB, "IDBServer::deleteDatabase - %s", requestData.databaseIdentifier().loggingString().utf8().data());
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto connection = m_connectionMap.get(requestData.requestIdentifier().connectionIdentifier());
+    auto connectionIdentifier = requestData.requestIdentifier().connectionIdentifier();
+    if (!connectionIdentifier)
+        return;
+
+    RefPtr connection = m_connectionMap.get(*connectionIdentifier);
     if (!connection) {
         // If the connection back to the client is gone, there's no way to delete the database as
         // well as no way to message back failure.
@@ -183,7 +189,7 @@ void IDBServer::abortTransaction(const IDBResourceIdentifier& transactionIdentif
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(transactionIdentifier);
+    RefPtr transaction = m_transactions.get(transactionIdentifier);
     if (!transaction) {
         // If there is no transaction there is nothing to abort.
         // We also have no access to a connection over which to message failure-to-abort.
@@ -193,13 +199,18 @@ void IDBServer::abortTransaction(const IDBResourceIdentifier& transactionIdentif
     transaction->abort();
 }
 
+UniqueIDBDatabaseTransaction* IDBServer::idbTransaction(const IDBRequestData& requestData) const
+{
+    return m_transactions.get(requestData.transactionIdentifier());
+}
+
 void IDBServer::createObjectStore(const IDBRequestData& requestData, const IDBObjectStoreInfo& info)
 {
     LOG(IndexedDB, "IDBServer::createObjectStore");
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -213,7 +224,7 @@ void IDBServer::deleteObjectStore(const IDBRequestData& requestData, const Strin
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -221,13 +232,13 @@ void IDBServer::deleteObjectStore(const IDBRequestData& requestData, const Strin
     transaction->deleteObjectStore(requestData, objectStoreName);
 }
 
-void IDBServer::renameObjectStore(const IDBRequestData& requestData, uint64_t objectStoreIdentifier, const String& newName)
+void IDBServer::renameObjectStore(const IDBRequestData& requestData, IDBObjectStoreIdentifier objectStoreIdentifier, const String& newName)
 {
     LOG(IndexedDB, "IDBServer::renameObjectStore");
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -235,13 +246,13 @@ void IDBServer::renameObjectStore(const IDBRequestData& requestData, uint64_t ob
     transaction->renameObjectStore(requestData, objectStoreIdentifier, newName);
 }
 
-void IDBServer::clearObjectStore(const IDBRequestData& requestData, uint64_t objectStoreIdentifier)
+void IDBServer::clearObjectStore(const IDBRequestData& requestData, IDBObjectStoreIdentifier objectStoreIdentifier)
 {
     LOG(IndexedDB, "IDBServer::clearObjectStore");
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -254,7 +265,7 @@ void IDBServer::createIndex(const IDBRequestData& requestData, const IDBIndexInf
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -262,13 +273,13 @@ void IDBServer::createIndex(const IDBRequestData& requestData, const IDBIndexInf
     transaction->createIndex(requestData, info);
 }
 
-void IDBServer::deleteIndex(const IDBRequestData& requestData, uint64_t objectStoreIdentifier, const String& indexName)
+void IDBServer::deleteIndex(const IDBRequestData& requestData, IDBObjectStoreIdentifier objectStoreIdentifier, const String& indexName)
 {
     LOG(IndexedDB, "IDBServer::deleteIndex");
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -276,13 +287,13 @@ void IDBServer::deleteIndex(const IDBRequestData& requestData, uint64_t objectSt
     transaction->deleteIndex(requestData, objectStoreIdentifier, indexName);
 }
 
-void IDBServer::renameIndex(const IDBRequestData& requestData, uint64_t objectStoreIdentifier, uint64_t indexIdentifier, const String& newName)
+void IDBServer::renameIndex(const IDBRequestData& requestData, IDBObjectStoreIdentifier objectStoreIdentifier, IDBIndexIdentifier indexIdentifier, const String& newName)
 {
     LOG(IndexedDB, "IDBServer::renameIndex");
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -290,17 +301,17 @@ void IDBServer::renameIndex(const IDBRequestData& requestData, uint64_t objectSt
     transaction->renameIndex(requestData, objectStoreIdentifier, indexIdentifier, newName);
 }
 
-void IDBServer::putOrAdd(const IDBRequestData& requestData, const IDBKeyData& keyData, const IDBValue& value, IndexedDB::ObjectStoreOverwriteMode overwriteMode)
+void IDBServer::putOrAdd(const IDBRequestData& requestData, const IDBKeyData& keyData, const IDBValue& value, const IndexIDToIndexKeyMap& indexKeys, IndexedDB::ObjectStoreOverwriteMode overwriteMode)
 {
     LOG(IndexedDB, "IDBServer::putOrAdd");
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
-    transaction->putOrAdd(requestData, keyData, value, overwriteMode);
+    transaction->putOrAdd(requestData, keyData, value, indexKeys, overwriteMode);
 }
 
 void IDBServer::getRecord(const IDBRequestData& requestData, const IDBGetRecordData& getRecordData)
@@ -309,7 +320,7 @@ void IDBServer::getRecord(const IDBRequestData& requestData, const IDBGetRecordD
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -322,7 +333,7 @@ void IDBServer::getAllRecords(const IDBRequestData& requestData, const IDBGetAll
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -335,7 +346,7 @@ void IDBServer::getCount(const IDBRequestData& requestData, const IDBKeyRangeDat
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -348,7 +359,7 @@ void IDBServer::deleteRecord(const IDBRequestData& requestData, const IDBKeyRang
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -361,7 +372,7 @@ void IDBServer::openCursor(const IDBRequestData& requestData, const IDBCursorInf
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
@@ -374,14 +385,14 @@ void IDBServer::iterateCursor(const IDBRequestData& requestData, const IDBIterat
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(requestData.transactionIdentifier());
+    RefPtr transaction = idbTransaction(requestData);
     if (!transaction)
         return;
 
     transaction->iterateCursor(requestData, data);
 }
 
-void IDBServer::establishTransaction(uint64_t databaseConnectionIdentifier, const IDBTransactionInfo& info)
+void IDBServer::establishTransaction(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier, const IDBTransactionInfo& info)
 {
     LOG(IndexedDB, "IDBServer::establishTransaction");
     ASSERT(!isMainThread());
@@ -397,23 +408,23 @@ void IDBServer::establishTransaction(uint64_t databaseConnectionIdentifier, cons
         m_uniqueIDBDatabaseMap.remove(database->identifier());
 }
 
-void IDBServer::commitTransaction(const IDBResourceIdentifier& transactionIdentifier, uint64_t pendingRequestCount)
+void IDBServer::commitTransaction(const IDBResourceIdentifier& transactionIdentifier, uint64_t handledRequestResultsCount)
 {
     LOG(IndexedDB, "IDBServer::commitTransaction");
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    auto transaction = m_transactions.get(transactionIdentifier);
+    RefPtr transaction = m_transactions.get(transactionIdentifier);
     if (!transaction) {
         // If there is no transaction there is nothing to commit.
         // We also have no access to a connection over which to message failure-to-commit.
         return;
     }
 
-    transaction->commit(pendingRequestCount);
+    transaction->commit(handledRequestResultsCount);
 }
 
-void IDBServer::didFinishHandlingVersionChangeTransaction(uint64_t databaseConnectionIdentifier, const IDBResourceIdentifier& transactionIdentifier)
+void IDBServer::didFinishHandlingVersionChangeTransaction(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier, const IDBResourceIdentifier& transactionIdentifier)
 {
     LOG(IndexedDB, "IDBServer::didFinishHandlingVersionChangeTransaction - %s", transactionIdentifier.loggingString().utf8().data());
     ASSERT(!isMainThread());
@@ -426,9 +437,9 @@ void IDBServer::didFinishHandlingVersionChangeTransaction(uint64_t databaseConne
     connection->didFinishHandlingVersionChange(transactionIdentifier);
 }
 
-void IDBServer::databaseConnectionPendingClose(uint64_t databaseConnectionIdentifier)
+void IDBServer::databaseConnectionPendingClose(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier)
 {
-    LOG(IndexedDB, "IDBServer::databaseConnectionPendingClose - %" PRIu64, databaseConnectionIdentifier);
+    LOG(IndexedDB, "IDBServer::databaseConnectionPendingClose - %" PRIu64, databaseConnectionIdentifier.toUInt64());
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
@@ -439,9 +450,9 @@ void IDBServer::databaseConnectionPendingClose(uint64_t databaseConnectionIdenti
     databaseConnection->connectionPendingCloseFromClient();
 }
 
-void IDBServer::databaseConnectionClosed(uint64_t databaseConnectionIdentifier)
+void IDBServer::databaseConnectionClosed(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier)
 {
-    LOG(IndexedDB, "IDBServer::databaseConnectionClosed - %" PRIu64, databaseConnectionIdentifier);
+    LOG(IndexedDB, "IDBServer::databaseConnectionClosed - %" PRIu64, databaseConnectionIdentifier.toUInt64());
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
@@ -455,14 +466,14 @@ void IDBServer::databaseConnectionClosed(uint64_t databaseConnectionIdentifier)
         m_uniqueIDBDatabaseMap.remove(database->identifier());
 }
 
-void IDBServer::abortOpenAndUpgradeNeeded(uint64_t databaseConnectionIdentifier, const std::optional<IDBResourceIdentifier>& transactionIdentifier)
+void IDBServer::abortOpenAndUpgradeNeeded(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier, const std::optional<IDBResourceIdentifier>& transactionIdentifier)
 {
     LOG(IndexedDB, "IDBServer::abortOpenAndUpgradeNeeded");
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
     if (transactionIdentifier) {
-        if (auto transaction = m_transactions.get(*transactionIdentifier))
+        if (RefPtr transaction = m_transactions.get(*transactionIdentifier))
             transaction->abortWithoutCallback();
     }
 
@@ -473,7 +484,7 @@ void IDBServer::abortOpenAndUpgradeNeeded(uint64_t databaseConnectionIdentifier,
     databaseConnection->connectionClosedFromClient();
 }
 
-void IDBServer::didFireVersionChangeEvent(uint64_t databaseConnectionIdentifier, const IDBResourceIdentifier& requestIdentifier, IndexedDB::ConnectionClosedOnBehalfOfServer connectionClosed)
+void IDBServer::didFireVersionChangeEvent(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier, const IDBResourceIdentifier& requestIdentifier, IndexedDB::ConnectionClosedOnBehalfOfServer connectionClosed)
 {
     LOG(IndexedDB, "IDBServer::didFireVersionChangeEvent");
     ASSERT(!isMainThread());
@@ -483,7 +494,7 @@ void IDBServer::didFireVersionChangeEvent(uint64_t databaseConnectionIdentifier,
         databaseConnection->didFireVersionChangeEvent(requestIdentifier, connectionClosed);
 }
 
-void IDBServer::openDBRequestCancelled(const IDBRequestData& requestData)
+void IDBServer::openDBRequestCancelled(const IDBOpenRequestData& requestData)
 {
     LOG(IndexedDB, "IDBServer::openDBRequestCancelled");
     ASSERT(!isMainThread());
@@ -537,7 +548,7 @@ void IDBServer::getAllDatabaseNamesAndVersions(IDBConnectionIdentifier serverCon
     auto directory = IDBDatabaseIdentifier::databaseDirectoryRelativeToRoot(origin, m_databaseDirectoryPath, "v1"_s);
     getDatabaseNameAndVersionFromOriginDirectory(directory, visitedDatabasePaths, result);
 
-    auto connection = m_connectionMap.get(serverConnectionIdentifier);
+    RefPtr connection = m_connectionMap.get(serverConnectionIdentifier);
     if (!connection)
         return;
 
@@ -745,14 +756,12 @@ void IDBServer::requestSpace(const ClientOrigin& origin, uint64_t taskSize, Comp
     ASSERT(!isMainThread());
     ASSERT(m_lock.isHeld());
 
-    StorageQuotaManager::Decision result = StorageQuotaManager::Decision::Deny;
-
     // Release lock because space requesting could be blocked.
     m_lock.unlock();
-    result = m_spaceRequester(origin, taskSize);
+    bool result = m_spaceRequester(origin, taskSize);
     m_lock.lock();
 
-    completionHandler(result == StorageQuotaManager::Decision::Grant);
+    completionHandler(result);
 }
 
 uint64_t IDBServer::diskUsage(const String& rootDirectory, const ClientOrigin& origin)
@@ -791,37 +800,5 @@ String IDBServer::upgradedDatabaseDirectory(const WebCore::IDBDatabaseIdentifier
     return newDatabaseDirectory;
 }
 
-bool IDBServer::hasDatabaseActivitiesOnMainThread() const
-{
-    ASSERT(isMainThread());
-    ASSERT(m_lock.isHeld());
-
-    if (m_sessionID.isEphemeral())
-        return false;
-
-    for (auto& database : m_uniqueIDBDatabaseMap.values()) {
-        if (!database->identifier().isTransient() && database->hasActiveTransactions())
-            return true;
-    }
-    
-    return false;
-}
-
-void IDBServer::stopDatabaseActivitiesOnMainThread()
-{
-    ASSERT(isMainThread());
-    ASSERT(m_lock.isHeld());
-
-    // Only stop non-ephemeral IDBServer that can hold database file lock.
-    if (m_sessionID.isEphemeral())
-        return;
-
-    for (auto& database : m_uniqueIDBDatabaseMap.values()) {
-        // Only stop databases with non-ephemeral backing store that can hold database file lock.
-        if (!database->identifier().isTransient())
-            database->abortActiveTransactions();
-    }
-}
-
 } // namespace IDBServer
 } // namespace WebCore
